@@ -1,4 +1,9 @@
 import traceback
+from collections import namedtuple
+from dataclasses import (
+    dataclass,
+    field,
+)
 from gettext import gettext
 from typing import (
     TYPE_CHECKING,
@@ -10,9 +15,8 @@ from typing import (
     Type,
 )
 
-import attr
-
 from .exceptions import SnapshotDoesNotExist
+from .extensions.amber.serializer import Repr
 
 if TYPE_CHECKING:
     from .extensions.base import AbstractSyrupyExtension
@@ -23,19 +27,20 @@ if TYPE_CHECKING:
         PropertyMatcher,
         SerializableData,
         SerializedData,
+        SnapshotIndex,
     )
 
 
-@attr.s
+@dataclass
 class AssertionResult:
-    snapshot_location: str = attr.ib()
-    snapshot_name: str = attr.ib()
-    asserted_data: Optional["SerializedData"] = attr.ib()
-    recalled_data: Optional["SerializedData"] = attr.ib()
-    created: bool = attr.ib()
-    updated: bool = attr.ib()
-    success: bool = attr.ib()
-    exception: Optional[Exception] = attr.ib()
+    snapshot_location: str
+    snapshot_name: str
+    asserted_data: Optional["SerializedData"]
+    recalled_data: Optional["SerializedData"]
+    created: bool
+    updated: bool
+    success: bool
+    exception: Optional[Exception]
 
     @property
     def final_data(self) -> Optional["SerializedData"]:
@@ -44,42 +49,57 @@ class AssertionResult:
         return self.recalled_data
 
 
-@attr.s(cmp=False, repr=False)
+@dataclass(eq=False, order=False, repr=False)
 class SnapshotAssertion:
-    name: str = attr.ib(default="snapshot")
-    _session: "SnapshotSession" = attr.ib(kw_only=True)
-    _extension_class: Type["AbstractSyrupyExtension"] = attr.ib(kw_only=True)
-    _test_location: "PyTestLocation" = attr.ib(kw_only=True)
-    _update_snapshots: bool = attr.ib(kw_only=True)
-    _exclude: Optional["PropertyFilter"] = attr.ib(
-        init=False, default=None, kw_only=True
+    session: "SnapshotSession"
+    extension_class: Type["AbstractSyrupyExtension"]
+    test_location: "PyTestLocation"
+    update_snapshots: bool
+
+    _exclude: Optional["PropertyFilter"] = field(
+        init=False,
+        default=None,
     )
-    _extension: Optional["AbstractSyrupyExtension"] = attr.ib(
-        init=False, default=None, kw_only=True
+    _custom_index: Optional[str] = field(
+        init=False,
+        default=None,
     )
-    _executions: int = attr.ib(init=False, default=0, kw_only=True)
-    _execution_results: Dict[int, "AssertionResult"] = attr.ib(
-        init=False, factory=dict, kw_only=True
+    _extension: Optional["AbstractSyrupyExtension"] = field(
+        init=False,
+        default=None,
     )
-    _matcher: Optional["PropertyMatcher"] = attr.ib(
-        init=False, default=None, kw_only=True
+    _executions: int = field(
+        init=False,
+        default=0,
     )
-    _post_assert_actions: List[Callable[..., None]] = attr.ib(
-        init=False, factory=list, kw_only=True
+    _execution_results: Dict[int, "AssertionResult"] = field(
+        init=False,
+        default_factory=dict,
+    )
+    _execution_name_index: Dict["SnapshotIndex", int] = field(
+        init=False, default_factory=dict
+    )
+    _matcher: Optional["PropertyMatcher"] = field(
+        init=False,
+        default=None,
+    )
+    _post_assert_actions: List[Callable[..., None]] = field(
+        init=False,
+        default_factory=list,
     )
 
-    def __attrs_post_init__(self) -> None:
-        self._session.register_request(self)
+    def __post_init__(self) -> None:
+        self.session.register_request(self)
 
     def __init_extension(
         self, extension_class: Type["AbstractSyrupyExtension"]
     ) -> "AbstractSyrupyExtension":
-        return extension_class(test_location=self._test_location)
+        return extension_class(test_location=self.test_location)
 
     @property
     def extension(self) -> "AbstractSyrupyExtension":
         if not self._extension:
-            self._extension = self.__init_extension(self._extension_class)
+            self._extension = self.__init_extension(self.extension_class)
         return self._extension
 
     @property
@@ -87,9 +107,53 @@ class SnapshotAssertion:
         return int(self._executions)
 
     @property
-    def executions(self) -> Dict[int, AssertionResult]:
+    def executions(self) -> Dict[int, "AssertionResult"]:
         return self._execution_results
 
+    @property
+    def index(self) -> "SnapshotIndex":
+        if self._custom_index:
+            return self._custom_index
+        return self.num_executions
+
+    @property
+    def name(self) -> str:
+        return self._custom_index or "snapshot"
+
+    @property
+    def __repr(self) -> "SerializableData":
+        SnapshotAssertionRepr = namedtuple(  # type: ignore
+            "SnapshotAssertion", ["name", "num_executions"]
+        )
+        assertion_result = self.executions.get(
+            (self._custom_index and self._execution_name_index.get(self._custom_index))
+            or self.num_executions - 1
+        )
+        return (
+            Repr(str(assertion_result.final_data))
+            if assertion_result
+            else SnapshotAssertionRepr(
+                name=self.name,
+                num_executions=self.num_executions,
+            )
+        )
+
+    @property
+    def __matcher(self) -> "PropertyMatcher":
+        """
+        Get matcher that replaces `SnapshotAssertion` with one that can be serialized
+        """
+
+        def _matcher(**kwargs: Any) -> Optional["SerializableData"]:
+            maybe_assertion = kwargs.get("data")
+            if isinstance(maybe_assertion, SnapshotAssertion):
+                return maybe_assertion.__repr
+            if self._matcher:
+                return self._matcher(**kwargs)
+            return maybe_assertion
+
+        return _matcher
+
     def use_extension(
         self, extension_class: Optional[Type["AbstractSyrupyExtension"]] = None
     ) -> "SnapshotAssertion":
@@ -98,10 +162,10 @@ class SnapshotAssertion:
         specified extension class. This does not preserve assertion index or state.
         """
         return self.__class__(
-            update_snapshots=self._update_snapshots,
-            test_location=self._test_location,
-            extension_class=extension_class or self._extension_class,
-            session=self._session,
+            update_snapshots=self.update_snapshots,
+            test_location=self.test_location,
+            extension_class=extension_class or self.extension_class,
+            session=self.session,
         )
 
     def assert_match(self, data: "SerializableData") -> None:
@@ -109,7 +173,7 @@ class SnapshotAssertion:
 
     def _serialize(self, data: "SerializableData") -> "SerializedData":
         return self.extension.serialize(
-            data, exclude=self._exclude, matcher=self._matcher
+            data, exclude=self._exclude, matcher=self.__matcher
         )
 
     def get_assert_diff(self) -> List[str]:
@@ -146,9 +210,11 @@ class SnapshotAssertion:
     def __call__(
         self,
         *,
+        diff: Optional["SnapshotIndex"] = None,
         exclude: Optional["PropertyFilter"] = None,
         extension_class: Optional[Type["AbstractSyrupyExtension"]] = None,
         matcher: Optional["PropertyMatcher"] = None,
+        name: Optional["SnapshotIndex"] = None,
     ) -> "SnapshotAssertion":
         """
         Modifies assertion instance options
@@ -159,32 +225,46 @@ class SnapshotAssertion:
             self.__with_prop("_extension", self.__init_extension(extension_class))
         if matcher:
             self.__with_prop("_matcher", matcher)
+        if name:
+            self.__with_prop("_custom_index", name)
+        if diff is not None:
+            self.__with_prop("_snapshot_diff", diff)
         return self
 
     def __repr__(self) -> str:
-        attrs_to_repr = ["name", "num_executions"]
-        attrs_repr = ", ".join(f"{a}={repr(getattr(self, a))}" for a in attrs_to_repr)
-        return f"SnapshotAssertion({attrs_repr})"
+        return str(self._serialize(self.__repr))
 
     def __eq__(self, other: "SerializableData") -> bool:
         return self._assert(other)
 
     def _assert(self, data: "SerializableData") -> bool:
-        snapshot_location = self.extension.get_location(index=self.num_executions)
-        snapshot_name = self.extension.get_snapshot_name(index=self.num_executions)
+        snapshot_location = self.extension.get_location(index=self.index)
+        snapshot_name = self.extension.get_snapshot_name(index=self.index)
         snapshot_data: Optional["SerializedData"] = None
         serialized_data: Optional["SerializedData"] = None
         matches = False
         assertion_success = False
         assertion_exception = None
         try:
-            snapshot_data = self._recall_data(index=self.num_executions)
+            snapshot_data = self._recall_data(index=self.index)
             serialized_data = self._serialize(data)
-            matches = snapshot_data is not None and serialized_data == snapshot_data
+            snapshot_diff = getattr(self, "_snapshot_diff", None)
+            if snapshot_diff is not None:
+                snapshot_data_diff = self._recall_data(index=snapshot_diff)
+                if snapshot_data_diff is None:
+                    raise SnapshotDoesNotExist()
+                serialized_data = self.extension.diff_snapshots(
+                    serialized_data=serialized_data,
+                    snapshot_data=snapshot_data_diff,
+                )
+            matches = snapshot_data is not None and self.extension.matches(
+                serialized_data=serialized_data, snapshot_data=snapshot_data
+            )
             assertion_success = matches
-            if not matches and self._update_snapshots:
+            if not matches and self.update_snapshots:
                 self.extension.write_snapshot(
-                    data=serialized_data, index=self.num_executions
+                    data=serialized_data,
+                    index=self.index,
                 )
                 assertion_success = True
             return assertion_success
@@ -194,6 +274,7 @@ class SnapshotAssertion:
         finally:
             snapshot_created = snapshot_data is None and assertion_success
             snapshot_updated = matches is False and assertion_success
+            self._execution_name_index[self.index] = self._executions
             self._execution_results[self._executions] = AssertionResult(
                 snapshot_location=snapshot_location,
                 snapshot_name=snapshot_name,
@@ -214,7 +295,7 @@ class SnapshotAssertion:
         while self._post_assert_actions:
             self._post_assert_actions.pop()()
 
-    def _recall_data(self, index: int) -> Optional["SerializableData"]:
+    def _recall_data(self, index: "SnapshotIndex") -> Optional["SerializableData"]:
         try:
             return self.extension.read_snapshot(index=index)
         except SnapshotDoesNotExist:
