@@ -1,16 +1,26 @@
+import functools
 import os
-from types import GeneratorType
+from types import (
+    GeneratorType,
+    MappingProxyType,
+)
 from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
+    Dict,
     Iterable,
+    NamedTuple,
     Optional,
     Set,
     Tuple,
+    Union,
 )
 
-from syrupy.constants import SYMBOL_ELLIPSIS
+from syrupy.constants import (
+    SYMBOL_ELLIPSIS,
+    TEXT_ENCODING,
+)
 from syrupy.data import (
     Snapshot,
     SnapshotFossil,
@@ -25,8 +35,10 @@ if TYPE_CHECKING:
         SerializableData,
     )
 
-    PropertyValueFilter = Callable[[SerializableData], bool]
-    PropertyValueGetter = Callable[..., SerializableData]
+    PropertyValueFilter = Callable[["PropertyName"], bool]
+    PropertyValueGetter = Callable[
+        ["SerializableData", "PropertyName"], "SerializableData"
+    ]
     IterableEntries = Tuple[
         Iterable["PropertyName"],
         "PropertyValueGetter",
@@ -42,11 +54,20 @@ class Repr:
         return self._repr
 
 
+def attr_getter(o: "SerializableData", p: "PropertyName") -> "SerializableData":
+    return getattr(o, str(p))
+
+
+def item_getter(o: "SerializableData", p: "PropertyName") -> "SerializableData":
+    return o[p]
+
+
 class DataSerializer:
     _indent: str = "  "
     _max_depth: int = 99
-    _marker_divider: str = "---"
-    _marker_name: str = "# name:"
+    _marker_comment: str = "# "
+    _marker_divider: str = f"{_marker_comment}---"
+    _marker_name: str = f"{_marker_comment}name:"
     _marker_crn: str = "\r\n"
 
     @classmethod
@@ -55,7 +76,7 @@ class DataSerializer:
         Writes the snapshot data into the snapshot file that be read later.
         """
         filepath = snapshot_fossil.location
-        with open(filepath, "w", encoding="utf-8", newline=None) as f:
+        with open(filepath, "w", encoding=TEXT_ENCODING, newline=None) as f:
             for snapshot in sorted(snapshot_fossil, key=lambda s: s.name):
                 snapshot_data = str(snapshot.data)
                 if snapshot_data is not None:
@@ -65,6 +86,7 @@ class DataSerializer:
                     f.write(f"\n{cls._marker_divider}\n")
 
     @classmethod
+    @functools.lru_cache()
     def read_file(cls, filepath: str) -> "SnapshotFossil":
         """
         Read the raw snapshot data (str) from the snapshot file into a dict
@@ -75,7 +97,7 @@ class DataSerializer:
         indent_len = len(cls._indent)
         snapshot_fossil = SnapshotFossil(location=filepath)
         try:
-            with open(filepath, "r", encoding="utf-8", newline=None) as f:
+            with open(filepath, "r", encoding=TEXT_ENCODING, newline=None) as f:
                 test_name = None
                 snapshot_data = ""
                 for line in f:
@@ -147,7 +169,7 @@ class DataSerializer:
             serialize_method = cls.serialize_number
         elif isinstance(data, (set, frozenset)):
             serialize_method = cls.serialize_set
-        elif isinstance(data, dict):
+        elif isinstance(data, (dict, MappingProxyType)):
             serialize_method = cls.serialize_dict
         elif cls.__is_namedtuple(data):
             serialize_method = cls.serialize_namedtuple
@@ -157,14 +179,12 @@ class DataSerializer:
 
     @classmethod
     def serialize_number(
-        cls, data: "SerializableData", *, depth: int = 0, **kwargs: Any
+        cls, data: Union[int, float], *, depth: int = 0, **kwargs: Any
     ) -> str:
         return cls.__serialize_plain(data=data, depth=depth)
 
     @classmethod
-    def serialize_string(
-        cls, data: "SerializableData", *, depth: int = 0, **kwargs: Any
-    ) -> str:
+    def serialize_string(cls, data: str, *, depth: int = 0, **kwargs: Any) -> str:
         if all(c not in data for c in cls._marker_crn):
             return cls.__serialize_plain(data=data, depth=depth)
 
@@ -175,60 +195,57 @@ class DataSerializer:
                 for line in str(data).splitlines(keepends=True)
             ),
             depth=depth,
-            open_tag="'",
-            close_tag="'",
+            open_tag="'''",
+            close_tag="'''",
             include_type=False,
             ends="",
         )
 
     @classmethod
-    def serialize_iterable(cls, data: "SerializableData", **kwargs: Any) -> str:
-        open_paren, close_paren = next(
-            parens
-            for iter_type, parens in {
-                GeneratorType: ("(", ")"),
-                list: ("[", "]"),
-                tuple: ("(", ")"),
-            }.items()
-            if isinstance(data, iter_type)
-        )
+    def serialize_iterable(
+        cls, data: Iterable["SerializableData"], **kwargs: Any
+    ) -> str:
+        open_paren, close_paren = (None, None)
+        if isinstance(data, list):
+            open_paren, close_paren = ("[", "]")
+
         values = list(data)
         return cls.__serialize_iterable(
             data=data,
-            resolve_entries=(range(len(values)), lambda _, p: values[p], None),
-            open_tag=open_paren,
-            close_tag=close_paren,
+            resolve_entries=(range(len(values)), item_getter, None),
+            open_paren=open_paren,
+            close_paren=close_paren,
             **kwargs,
         )
 
     @classmethod
-    def serialize_set(cls, data: "SerializableData", **kwargs: Any) -> str:
+    def serialize_set(cls, data: Set["SerializableData"], **kwargs: Any) -> str:
         return cls.__serialize_iterable(
             data=data,
             resolve_entries=(cls.sort(data), lambda _, p: p, None),
-            open_tag="{",
-            close_tag="}",
+            open_paren="{",
+            close_paren="}",
             **kwargs,
         )
 
     @classmethod
-    def serialize_namedtuple(cls, data: "SerializableData", **kwargs: Any) -> str:
+    def serialize_namedtuple(cls, data: NamedTuple, **kwargs: Any) -> str:
         return cls.__serialize_iterable(
             data=data,
-            resolve_entries=(cls.sort(data._fields), getattr, None),
-            open_tag="(",
-            close_tag=")",
+            resolve_entries=(cls.sort(data._fields), attr_getter, None),
             separator="=",
             **kwargs,
         )
 
     @classmethod
-    def serialize_dict(cls, data: "SerializableData", **kwargs: Any) -> str:
+    def serialize_dict(
+        cls, data: Dict["PropertyName", "SerializableData"], **kwargs: Any
+    ) -> str:
         return cls.__serialize_iterable(
             data=data,
-            resolve_entries=(cls.sort(data.keys()), lambda d, p: d[p], None),
-            open_tag="{",
-            close_tag="}",
+            resolve_entries=(cls.sort(data.keys()), item_getter, None),
+            open_paren="{",
+            close_paren="}",
             separator=": ",
             serialize_key=True,
             **kwargs,
@@ -243,12 +260,10 @@ class DataSerializer:
             data=data,
             resolve_entries=(
                 (name for name in cls.sort(dir(data)) if not name.startswith("_")),
-                getattr,
+                attr_getter,
                 lambda v: not callable(v),
             ),
             depth=depth,
-            open_tag="{",
-            close_tag="}",
             separator="=",
             **kwargs,
         )
@@ -266,7 +281,7 @@ class DataSerializer:
 
     @classmethod
     def object_type(cls, data: "SerializableData") -> str:
-        return f"<class '{data.__class__.__name__}'>"
+        return f"{data.__class__.__name__}"
 
     @classmethod
     def __is_namedtuple(cls, obj: Any) -> bool:
@@ -289,8 +304,8 @@ class DataSerializer:
         *,
         data: "SerializableData",
         resolve_entries: "IterableEntries",
-        open_tag: str,
-        close_tag: str,
+        open_paren: Optional[str] = None,
+        close_paren: Optional[str] = None,
         depth: int = 0,
         exclude: Optional["PropertyFilter"] = None,
         path: "PropertyPath" = (),
@@ -331,8 +346,8 @@ class DataSerializer:
             data=data,
             lines=(f"{key_str(key)}{value_str(key, value)}," for key, value in entries),
             depth=depth,
-            open_tag=open_tag,
-            close_tag=close_tag,
+            open_tag=f"({open_paren or ''}",
+            close_tag=f"{close_paren or ''})",
         )
 
     @classmethod
@@ -349,7 +364,7 @@ class DataSerializer:
     ) -> str:
         lines = ends.join(lines)
         lines_end = "\n" if lines else ""
-        maybe_obj_type = f"{cls.object_type(data)} " if include_type else ""
+        maybe_obj_type = f"{cls.object_type(data)}" if include_type else ""
         formatted_open_tag = cls.with_indent(f"{maybe_obj_type}{open_tag}", depth)
         formatted_close_tag = cls.with_indent(close_tag, depth)
         return f"{formatted_open_tag}\n{lines}{lines_end}{formatted_close_tag}"
