@@ -18,6 +18,7 @@ from typing import (
 )
 
 from syrupy.constants import (
+    DISABLE_COLOR_ENV_VAR,
     SNAPSHOT_DIRNAME,
     SYMBOL_CARRIAGE,
     SYMBOL_ELLIPSIS,
@@ -39,7 +40,11 @@ from syrupy.terminal import (
     snapshot_diff_style,
     snapshot_style,
 )
-from syrupy.utils import walk_snapshot_dir
+from syrupy.utils import (
+    env_context,
+    obj_attrs,
+    walk_snapshot_dir,
+)
 
 if TYPE_CHECKING:
     from syrupy.location import PyTestLocation
@@ -48,6 +53,7 @@ if TYPE_CHECKING:
         PropertyMatcher,
         SerializableData,
         SerializedData,
+        SnapshotIndex,
     )
 
 
@@ -73,12 +79,16 @@ class SnapshotFossilizer(ABC):
     def test_location(self) -> "PyTestLocation":
         raise NotImplementedError
 
-    def get_snapshot_name(self, *, index: int = 0) -> str:
+    def get_snapshot_name(self, *, index: "SnapshotIndex" = 0) -> str:
         """Get the snapshot name for the assertion index in a test location"""
-        index_suffix = f".{index}" if index > 0 else ""
+        index_suffix = ""
+        if isinstance(index, (str,)):
+            index_suffix = f"[{index}]"
+        elif index:
+            index_suffix = f".{index}"
         return f"{self.test_location.snapshot_name}{index_suffix}"
 
-    def get_location(self, *, index: int) -> str:
+    def get_location(self, *, index: "SnapshotIndex") -> str:
         """Returns full location where snapshot data is stored."""
         basename = self._get_file_basename(index=index)
         fileext = f".{self._file_extension}" if self._file_extension else ""
@@ -105,7 +115,7 @@ class SnapshotFossilizer(ABC):
 
         return discovered
 
-    def read_snapshot(self, *, index: int) -> "SerializedData":
+    def read_snapshot(self, *, index: "SnapshotIndex") -> "SerializedData":
         """
         Utility method for reading the contents of a snapshot assertion.
         Will call `_pre_read`, then perform `read` and finally `post_read`,
@@ -127,7 +137,7 @@ class SnapshotFossilizer(ABC):
         finally:
             self._post_read(index=index)
 
-    def write_snapshot(self, *, data: "SerializedData", index: int) -> None:
+    def write_snapshot(self, *, data: "SerializedData", index: "SnapshotIndex") -> None:
         """
         Utility method for writing the contents of a snapshot assertion.
         Will call `_pre_write`, then perform `write` and finally `_post_write`.
@@ -173,16 +183,18 @@ class SnapshotFossilizer(ABC):
         """
         raise NotImplementedError
 
-    def _pre_read(self, *, index: int = 0) -> None:
+    def _pre_read(self, *, index: "SnapshotIndex" = 0) -> None:
         pass
 
-    def _post_read(self, *, index: int = 0) -> None:
+    def _post_read(self, *, index: "SnapshotIndex" = 0) -> None:
         pass
 
-    def _pre_write(self, *, data: "SerializedData", index: int = 0) -> None:
+    def _pre_write(self, *, data: "SerializedData", index: "SnapshotIndex" = 0) -> None:
         self.__ensure_snapshot_dir(index=index)
 
-    def _post_write(self, *, data: "SerializedData", index: int = 0) -> None:
+    def _post_write(
+        self, *, data: "SerializedData", index: "SnapshotIndex" = 0
+    ) -> None:
         pass
 
     @abstractmethod
@@ -218,11 +230,11 @@ class SnapshotFossilizer(ABC):
     def _file_extension(self) -> str:
         raise NotImplementedError
 
-    def _get_file_basename(self, *, index: int) -> str:
+    def _get_file_basename(self, *, index: "SnapshotIndex") -> str:
         """Returns file basename without extension. Used to create full filepath."""
         return self.test_location.filename
 
-    def __ensure_snapshot_dir(self, *, index: int) -> None:
+    def __ensure_snapshot_dir(self, *, index: "SnapshotIndex") -> None:
         """
         Ensures the folder path for the snapshot file exists.
         """
@@ -233,6 +245,16 @@ class SnapshotFossilizer(ABC):
 
 
 class SnapshotReporter(ABC):
+    _context_line_count = 1
+
+    def diff_snapshots(
+        self, serialized_data: "SerializedData", snapshot_data: "SerializedData"
+    ) -> "SerializedData":
+        env = {DISABLE_COLOR_ENV_VAR: "true"}
+        attrs = {"_context_line_count": 0}
+        with env_context(**env), obj_attrs(self, attrs):
+            return "\n".join(self.diff_lines(serialized_data, snapshot_data))
+
     def diff_lines(
         self, serialized_data: "SerializedData", snapshot_data: "SerializedData"
     ) -> Iterator[str]:
@@ -243,10 +265,6 @@ class SnapshotReporter(ABC):
     def _ends(self) -> Dict[str, str]:
         return {"\n": self._marker_new_line, "\r": self._marker_carriage}
 
-    @property
-    def _context_line_count(self) -> int:
-        return 1
-
     @property
     def _context_line_max(self) -> int:
         return self._context_line_count * 2
@@ -266,15 +284,16 @@ class SnapshotReporter(ABC):
     def __diff_lines(self, a: str, b: str) -> Iterator[str]:
         for line in self.__diffed_lines(a, b):
             show_ends = (
-                self.__strip_ends(line.a[1:]) == self.__strip_ends(line.b[1:])
+                self.__strip_ends(line.a[1:] if line.a is not None else "")
+                == self.__strip_ends(line.b[1:] if line.b is not None else "")
                 if line.is_complete
                 else False
             )
-            if line.has_snapshot:
+            if line.has_snapshot and line.a is not None:
                 yield self.__format_line(
                     line.a, line.diff_a, snapshot_style, snapshot_diff_style, show_ends
                 )
-            if line.has_received:
+            if line.has_received and line.b is not None:
                 yield self.__format_line(
                     line.b, line.diff_b, received_style, received_diff_style, show_ends
                 )
@@ -371,7 +390,23 @@ class SnapshotReporter(ABC):
         return line.rstrip("".join(self._ends.keys()))
 
 
-class AbstractSyrupyExtension(SnapshotSerializer, SnapshotFossilizer, SnapshotReporter):
+class SnapshotComparator(ABC):
+    def matches(
+        self,
+        *,
+        serialized_data: "SerializableData",
+        snapshot_data: "SerializableData",
+    ) -> bool:
+        """
+        Compares serialized data and snapshot data and returns
+        whether they match.
+        """
+        return bool(serialized_data == snapshot_data)
+
+
+class AbstractSyrupyExtension(
+    SnapshotSerializer, SnapshotFossilizer, SnapshotReporter, SnapshotComparator
+):
     def __init__(self, test_location: "PyTestLocation"):
         self._test_location = test_location
 
