@@ -1,13 +1,16 @@
+from enum import Enum
 from gettext import gettext
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
-    Any,
     Optional,
     Set,
+    Type,
+    Union,
 )
 from unicodedata import category
 
+from syrupy.constants import TEXT_ENCODING
 from syrupy.data import (
     Snapshot,
     SnapshotFossil,
@@ -16,14 +19,37 @@ from syrupy.data import (
 from .base import AbstractSyrupyExtension
 
 if TYPE_CHECKING:
-    from syrupy.types import SerializableData
+    from syrupy.types import (
+        PropertyFilter,
+        PropertyMatcher,
+        SerializableData,
+        SerializedData,
+        SnapshotIndex,
+    )
+
+
+class WriteMode(Enum):
+    BINARY = "b"
+    TEXT = "t"
+
+    def __str__(self) -> str:
+        return self.value
 
 
 class SingleFileSnapshotExtension(AbstractSyrupyExtension):
-    def serialize(self, data: "SerializableData", **kwargs: Any) -> bytes:
-        return bytes(data)
+    _text_encoding = TEXT_ENCODING
+    _write_mode = WriteMode.BINARY
 
-    def get_snapshot_name(self, *, index: int = 0) -> str:
+    def serialize(
+        self,
+        data: "SerializableData",
+        *,
+        exclude: Optional["PropertyFilter"] = None,
+        matcher: Optional["PropertyMatcher"] = None,
+    ) -> "SerializedData":
+        return self._supported_dataclass(data)
+
+    def get_snapshot_name(self, *, index: "SnapshotIndex" = 0) -> str:
         return self.__clean_filename(
             super(SingleFileSnapshotExtension, self).get_snapshot_name(index=index)
         )
@@ -37,7 +63,7 @@ class SingleFileSnapshotExtension(AbstractSyrupyExtension):
     def _file_extension(self) -> str:
         return "raw"
 
-    def _get_file_basename(self, *, index: int) -> str:
+    def _get_file_basename(self, *, index: "SnapshotIndex") -> str:
         return self.get_snapshot_name(index=index)
 
     @property
@@ -54,17 +80,37 @@ class SingleFileSnapshotExtension(AbstractSyrupyExtension):
         self, *, snapshot_location: str, snapshot_name: str
     ) -> Optional["SerializableData"]:
         try:
-            with open(snapshot_location, "rb") as f:
+            with open(
+                snapshot_location, f"r{self._write_mode}", encoding=self._write_encoding
+            ) as f:
                 return f.read()
         except FileNotFoundError:
             return None
 
+    @property
+    def _supported_dataclass(self) -> Union[Type[str], Type[bytes]]:
+        if self._write_mode == WriteMode.TEXT:
+            return str
+        return bytes
+
+    @property
+    def _write_encoding(self) -> Optional[str]:
+        if self._write_mode == WriteMode.TEXT:
+            return TEXT_ENCODING
+        return None
+
     def _write_snapshot_fossil(self, *, snapshot_fossil: "SnapshotFossil") -> None:
         filepath, data = snapshot_fossil.location, next(iter(snapshot_fossil)).data
-        if not isinstance(data, bytes):
-            error_text = gettext("Can write non binary data. Expected '{}', got '{}'")
-            raise TypeError(error_text.format(bytes.__name__, type(data).__name__))
-        with open(filepath, "wb") as f:
+        if not isinstance(data, self._supported_dataclass):
+            error_text = gettext(
+                "Can't write non supported data. Expected '{}', got '{}'"
+            )
+            raise TypeError(
+                error_text.format(
+                    self._supported_dataclass.__name__, type(data).__name__
+                )
+            )
+        with open(filepath, f"w{self._write_mode}", encoding=self._write_encoding) as f:
             f.write(data)
 
     def __clean_filename(self, filename: str) -> str:
