@@ -13,7 +13,7 @@ def paths(path_string: str, *path_strings: str) -> "PropertyFilter":
     Factory to create a filter using list of paths
     """
 
-    def path_filter(prop: "PropertyName", path: "PropertyPath") -> bool:
+    def path_filter(*, prop: "PropertyName", path: "PropertyPath") -> bool:
         path_str = ".".join(str(p) for p, _ in (*path, (prop, None)))
         return any(path_str == p for p in (path_string, *path_strings))
 
@@ -25,7 +25,7 @@ def props(prop_name: str, *prop_names: str) -> "PropertyFilter":
     Factory to create filter using list of props
     """
 
-    def prop_filter(prop: "PropertyName", path: "PropertyPath") -> bool:
+    def prop_filter(*, prop: "PropertyName", path: "PropertyPath") -> bool:
         return any(str(prop) == p for p in (prop_name, *prop_names))
 
     return prop_filter
