@@ -1,40 +1,69 @@
+from dataclasses import (
+    dataclass,
+    field,
+)
 from pathlib import Path
 from typing import (
     Iterator,
     Optional,
 )
 
-import attr
 import pytest
 
 from syrupy.constants import PYTEST_NODE_SEP
 
 
-@attr.s
+@dataclass
 class PyTestLocation:
-    _node: "pytest.Item" = attr.ib()
-    nodename: str = attr.ib(init=False)
-    testname: str = attr.ib(init=False)
-    methodname: str = attr.ib(init=False)
-    modulename: str = attr.ib(init=False)
-    filepath: str = attr.ib(init=False)
+    _node: "pytest.Item"
+    nodename: Optional[str] = field(init=False)
+    testname: str = field(init=False)
+    methodname: str = field(init=False)
+    modulename: str = field(init=False)
+    filepath: str = field(init=False)
 
-    def __attrs_post_init__(self) -> None:
-        self.filepath = getattr(self._node, "fspath", None)
-        obj = getattr(self._node, "obj", None)
+    def __post_init__(self) -> None:
+        if self.is_doctest:
+            return self.__attrs_post_init_doc__()
+        self.__attrs_post_init_def__()
+
+    def __attrs_post_init_def__(self) -> None:
+        self.filepath = getattr(self._node, "fspath")  # noqa: B009
+        obj = getattr(self._node, "obj")  # noqa: B009
         self.modulename = obj.__module__
         self.methodname = obj.__name__
         self.nodename = getattr(self._node, "name", None)
         self.testname = self.nodename or self.methodname
 
+    def __attrs_post_init_doc__(self) -> None:
+        doctest = getattr(self._node, "dtest")  # noqa: B009
+        self.filepath = doctest.filename
+        test_relfile, test_node = self.nodeid.split(PYTEST_NODE_SEP)
+        test_relpath = Path(test_relfile)
+        self.modulename = ".".join([*test_relpath.parent.parts, test_relpath.stem])
+        self.nodename = test_node.replace(f"{self.modulename}.", "")
+        self.testname = self.nodename or self.methodname
+
     @property
     def classname(self) -> Optional[str]:
+        if self.is_doctest:
+            return None
+        return ".".join(self.nodeid.split(PYTEST_NODE_SEP)[1:-1]) or None
+
+    @property
+    def nodeid(self) -> str:
         """
         Pytest node names contain file path and module members delimited by `::`
-        Example tests/grouping/test_file.py::TestClass::TestSubClass::test_method
+
+        Examples:
+        - tests/grouping/test_file.py::TestClass::TestSubClass::test_method
+        - tests/grouping/test_file.py::DocTestClass.doc_test_method
+        - tests/grouping/test_file.py::doctestfile.txt
+
+        :raises: `AttributeError` if node has no node id
+        :return: test node id
         """
-        nodeid: str = getattr(self._node, "nodeid", None)
-        return ".".join(nodeid.split(PYTEST_NODE_SEP)[1:-1]) or None
+        return str(getattr(self._node, "nodeid"))  # noqa: B009
 
     @property
     def filename(self) -> str:
@@ -46,6 +75,13 @@ class PyTestLocation:
             return f"{self.classname}.{self.testname}"
         return str(self.testname)
 
+    @property
+    def is_doctest(self) -> bool:
+        return self.__is_doctest(self._node)
+
+    def __is_doctest(self, node: "pytest.Item") -> bool:
+        return hasattr(node, "dtest")
+
     def __valid_id(self, name: str) -> str:
         """
         Take characters from the name while the result would be a valid python
@@ -78,4 +114,7 @@ class PyTestLocation:
         return self.__parse(self.snapshot_name) == self.__parse(snapshot_name)
 
     def matches_snapshot_location(self, snapshot_location: str) -> bool:
-        return self.filename in snapshot_location
+        loc = Path(snapshot_location)
+        # "test_file" should match_"test_file.ext" or "test_file/whatever.ext", but not
+        # "test_file_suffix.ext"
+        return self.filename == loc.stem or self.filename == loc.parent.name
