@@ -1,4 +1,9 @@
 import importlib
+from collections import defaultdict
+from dataclasses import (
+    dataclass,
+    field,
+)
 from gettext import (
     gettext,
     ngettext,
@@ -6,18 +11,16 @@ from gettext import (
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
+    Any,
     Callable,
+    DefaultDict,
     Dict,
     FrozenSet,
     Iterator,
     List,
-    Optional,
     Set,
-    Tuple,
 )
 
-import attr
-
 from .constants import PYTEST_NODE_SEP
 from .data import (
     Snapshot,
@@ -35,12 +38,14 @@ from .terminal import (
 )
 
 if TYPE_CHECKING:
+    import argparse
+
     import pytest
 
     from .assertion import SnapshotAssertion
 
 
-@attr.s
+@dataclass
 class SnapshotReport:
     """
     This class is responsible for determining the test summary and post execution
@@ -48,33 +53,50 @@ class SnapshotReport:
     information used for removal of unused or orphaned snapshots and fossils.
     """
 
-    base_dir: str = attr.ib()
-    collected_items: Set["pytest.Item"] = attr.ib()
-    selected_items: Dict[str, bool] = attr.ib()
-    update_snapshots: bool = attr.ib()
-    warn_unused_snapshots: bool = attr.ib()
-    include_snapshot_details: bool = attr.ib()
-    assertions: List["SnapshotAssertion"] = attr.ib()
-    discovered: "SnapshotFossils" = attr.ib(factory=SnapshotFossils)
-    created: "SnapshotFossils" = attr.ib(factory=SnapshotFossils)
-    failed: "SnapshotFossils" = attr.ib(factory=SnapshotFossils)
-    matched: "SnapshotFossils" = attr.ib(factory=SnapshotFossils)
-    updated: "SnapshotFossils" = attr.ib(factory=SnapshotFossils)
-    used: "SnapshotFossils" = attr.ib(factory=SnapshotFossils)
-    _invocation_args: Tuple[str, ...] = attr.ib(factory=tuple)
-    _provided_test_paths: Dict[str, List[str]] = attr.ib(factory=dict)
-    _keyword_expressions: Set["Expression"] = attr.ib(factory=set)
-    _collected_items_by_nodeid: Dict[str, "pytest.Item"] = attr.ib(
-        factory=dict, init=False
+    base_dir: str
+    collected_items: Set["pytest.Item"]
+    selected_items: Dict[str, bool]
+    options: "argparse.Namespace"
+    assertions: List["SnapshotAssertion"]
+    discovered: "SnapshotFossils" = field(default_factory=SnapshotFossils)
+    created: "SnapshotFossils" = field(default_factory=SnapshotFossils)
+    failed: "SnapshotFossils" = field(default_factory=SnapshotFossils)
+    matched: "SnapshotFossils" = field(default_factory=SnapshotFossils)
+    updated: "SnapshotFossils" = field(default_factory=SnapshotFossils)
+    used: "SnapshotFossils" = field(default_factory=SnapshotFossils)
+    _provided_test_paths: Dict[str, List[str]] = field(default_factory=dict)
+    _keyword_expressions: Set["Expression"] = field(default_factory=set)
+    _collected_items_by_nodeid: Dict[str, "pytest.Item"] = field(
+        default_factory=dict, init=False
     )
 
-    def __attrs_post_init__(self) -> None:
+    @property
+    def update_snapshots(self) -> bool:
+        return bool(self.options.update_snapshots)
+
+    @property
+    def warn_unused_snapshots(self) -> bool:
+        return bool(self.options.warn_unused_snapshots)
+
+    @property
+    def include_snapshot_details(self) -> bool:
+        return bool(self.options.include_snapshot_details)
+
+    def __post_init__(self) -> None:
         self.__parse_invocation_args()
         self._collected_items_by_nodeid = {
-            getattr(item, "nodeid", None): item for item in self.collected_items
+            getattr(item, "nodeid"): item for item in self.collected_items  # noqa: B009
         }
+
+        # We only need to discover snapshots once per test file, not once per assertion.
+        locations_discovered: DefaultDict[str, Set[Any]] = defaultdict(set)
         for assertion in self.assertions:
-            self.discovered.merge(assertion.extension.discover_snapshots())
+            test_location = assertion.extension.test_location.filepath
+            extension_class = assertion.extension.__class__
+            if extension_class not in locations_discovered[test_location]:
+                locations_discovered[test_location].add(extension_class)
+                self.discovered.merge(assertion.extension.discover_snapshots())
+
             for result in assertion.executions.values():
                 snapshot_fossil = SnapshotFossil(location=result.snapshot_location)
                 snapshot_fossil.add(
@@ -106,39 +128,24 @@ class SnapshotReport:
         would result in `"tests/test_file.py"` being stored as the location in a
         dictionary with `["TestClass", "test_method"]` being the test node path
         """
-        arg_groups: List[Tuple[Optional[str], str]] = []
-        path_as_package = False
-        maybe_opt_arg = None
-        for arg in self._invocation_args:
-            if arg.strip() == "--pyargs":
-                path_as_package = True
-            elif arg.startswith("-"):
-                if "=" in arg:
-                    arg0, arg1 = arg.split("=")
-                    arg_groups.append((arg0.strip(), arg1.strip()))
-                elif maybe_opt_arg is None:
-                    maybe_opt_arg = arg
-                    continue  # do not reset maybe_opt_arg
-            else:
-                arg_groups.append((maybe_opt_arg, arg.strip()))
 
-            maybe_opt_arg = None
-
-        for maybe_opt_arg, arg_value in arg_groups:
-            if maybe_opt_arg == "-k":  # or maybe_opt_arg == "-m":
-                self._keyword_expressions.add(Expression.compose(arg_value))
-            elif maybe_opt_arg is None:
-                parts = arg_value.split(PYTEST_NODE_SEP)
-                package_or_filepath = parts[0].strip()
-                filepath = Path(
-                    importlib.import_module(package_or_filepath).__file__
-                    if path_as_package
-                    else package_or_filepath
-                )
-                filepath_abs = str(
-                    filepath if filepath.is_absolute() else filepath.absolute()
-                )
-                self._provided_test_paths[filepath_abs] = parts[1:]
+        if self.options.keyword:
+            self._keyword_expressions.add(Expression.compose(self.options.keyword))
+        for file_or_dir in self.options.file_or_dir:
+            parts = file_or_dir.split(PYTEST_NODE_SEP)
+            package_or_filepath = parts[0].strip()
+            filepath = Path(package_or_filepath)
+            if self.options.pyargs:
+                try:
+                    mod = importlib.import_module(package_or_filepath)
+                    if mod.__file__ is not None:
+                        filepath = Path(mod.__file__)
+                except Exception:
+                    pass
+            filepath_abs = str(
+                filepath if filepath.is_absolute() else filepath.absolute()
+            )
+            self._provided_test_paths[filepath_abs] = parts[1:]
 
     @property
     def num_created(self) -> int:
@@ -204,8 +211,14 @@ class SnapshotReport:
                 unused_snapshots = {
                     snapshot
                     for snapshot in unused_snapshot_fossil
-                    if self._selected_items_match_name(snapshot.name)
-                    or self._provided_nodes_match_name(snapshot.name, provided_nodes)
+                    if self._selected_items_match_name(
+                        snapshot_location=snapshot_location, snapshot_name=snapshot.name
+                    )
+                    and self._provided_nodes_match_name(
+                        snapshot_location=snapshot_location,
+                        snapshot_name=snapshot.name,
+                        provided_nodes=provided_nodes,
+                    )
                 }
                 mark_for_removal = False
 
@@ -286,7 +299,14 @@ class SnapshotReport:
                 for snapshot_fossil in self.unused:
                     filepath = snapshot_fossil.location
                     snapshots = (snapshot.name for snapshot in snapshot_fossil)
-                    path_to_file = str(Path(filepath).relative_to(self.base_dir))
+
+                    try:
+                        path_to_file = str(Path(filepath).relative_to(self.base_dir))
+                    except ValueError:
+                        # this is just used for display, so better to fallback to
+                        # something vaguely reasonable (the full path) than give up
+                        path_to_file = filepath
+
                     unused_snapshots = ", ".join(map(bold, sorted(snapshots)))
                     yield warning_style(gettext(base_message)) + " {} ({})".format(
                         unused_snapshots, path_to_file
@@ -349,12 +369,22 @@ class SnapshotReport:
         ]
 
     def _provided_nodes_match_name(
-        self, snapshot_name: str, provided_nodes: List[List[str]]
+        self,
+        snapshot_location: str,
+        snapshot_name: str,
+        provided_nodes: List[List[str]],
     ) -> bool:
         """
-        Check that a snapshot name matches the node paths provided
+        Check that a snapshot name matches the node paths provided.
+        If no nodes are filtered, provided_nodes is empty, which means
+        all nodes should be matched.
         """
-        return any(snapshot_name in ".".join(node_path) for node_path in provided_nodes)
+        if not provided_nodes:
+            return True
+        for node_path in provided_nodes:
+            if snapshot_name in ".".join(node_path):
+                return True
+        return False
 
     def _provided_keywords_match_name(self, snapshot_name: str) -> bool:
         """
@@ -367,23 +397,30 @@ class SnapshotReport:
             for expr in self._keyword_expressions
         )
 
-    def _ran_items_match_name(self, snapshot_name: str) -> bool:
+    def _ran_items_match_name(self, snapshot_location: str, snapshot_name: str) -> bool:
         """
         Check that a snapshot name would match a test node using the Pytest location
         """
-        return any(
-            PyTestLocation(item).matches_snapshot_name(snapshot_name)
-            for item in self.ran_items
-        )
+        for item in self.ran_items:
+            location = PyTestLocation(item)
+            if location.matches_snapshot_location(
+                snapshot_location
+            ) and location.matches_snapshot_name(snapshot_name):
+                return True
+        return False
 
-    def _selected_items_match_name(self, snapshot_name: str) -> bool:
+    def _selected_items_match_name(
+        self, snapshot_location: str, snapshot_name: str
+    ) -> bool:
         """
         Check that a snapshot name should be treated as selected by the current session
         This being true means that if the snapshot was not used then it will be deleted
         """
         if self._keyword_expressions:
             return self._provided_keywords_match_name(snapshot_name)
-        return self._ran_items_match_name(snapshot_name)
+        return self._ran_items_match_name(
+            snapshot_location=snapshot_location, snapshot_name=snapshot_name
+        )
 
     def _ran_items_match_location(self, snapshot_location: str) -> bool:
         """
@@ -397,7 +434,7 @@ class SnapshotReport:
         )
 
 
-@attr.s(frozen=True)
+@dataclass(frozen=True)
 class Expression:
     """
     Dumbed down version of _pytest.mark.expression.Expression not available in < 6.0
@@ -406,7 +443,7 @@ class Expression:
     module is not public. This only supports inclusion based on simple string matching.
     """
 
-    code: FrozenSet[str] = attr.ib(factory=frozenset)
+    code: FrozenSet[str] = field(default_factory=frozenset)
 
     def evaluate(self, matcher: Callable[[str], bool]) -> bool:
         return any(map(matcher, self.code))
