@@ -1,15 +1,20 @@
+from collections import defaultdict
+from dataclasses import (
+    dataclass,
+    field,
+)
 from pathlib import Path
 from typing import (
     TYPE_CHECKING,
+    Any,
+    DefaultDict,
     Dict,
     Iterable,
     List,
     Optional,
     Set,
-    Tuple,
 )
 
-import attr
 import pytest
 
 from .constants import EXIT_STATUS_FAIL_UNUSED
@@ -21,27 +26,37 @@ if TYPE_CHECKING:
     from .extensions.base import AbstractSyrupyExtension
 
 
-@attr.s
+@dataclass
 class SnapshotSession:
-    base_dir: str = attr.ib()
-    update_snapshots: bool = attr.ib()
-    warn_unused_snapshots: bool = attr.ib()
-    include_snapshot_details: bool = attr.ib()
-    _invocation_args: Tuple[str, ...] = attr.ib(factory=tuple)
-    report: Optional["SnapshotReport"] = attr.ib(default=None)
+    # pytest.Session
+    pytest_session: Any
+    # Snapshot report generated on finish
+    report: Optional["SnapshotReport"] = None
     # All the collected test items
-    _collected_items: Set["pytest.Item"] = attr.ib(factory=set)
+    _collected_items: Set["pytest.Item"] = field(default_factory=set)
     # All the selected test items. Will be set to False until the test item is run.
-    _selected_items: Dict[str, bool] = attr.ib(factory=dict)
-    _assertions: List["SnapshotAssertion"] = attr.ib(factory=list)
-    _extensions: Dict[str, "AbstractSyrupyExtension"] = attr.ib(factory=dict)
+    _selected_items: Dict[str, bool] = field(default_factory=dict)
+    _assertions: List["SnapshotAssertion"] = field(default_factory=list)
+    _extensions: Dict[str, "AbstractSyrupyExtension"] = field(default_factory=dict)
+
+    _locations_discovered: DefaultDict[str, Set[Any]] = field(
+        default_factory=lambda: defaultdict(set)
+    )
+
+    @property
+    def update_snapshots(self) -> bool:
+        return bool(self.pytest_session.config.option.update_snapshots)
+
+    @property
+    def warn_unused_snapshots(self) -> bool:
+        return bool(self.pytest_session.config.option.warn_unused_snapshots)
 
     def collect_items(self, items: List["pytest.Item"]) -> None:
         self._collected_items.update(self.filter_valid_items(items))
 
     def select_items(self, items: List["pytest.Item"]) -> None:
         for item in self.filter_valid_items(items):
-            self._selected_items[getattr(item, "nodeid", None)] = False
+            self._selected_items[getattr(item, "nodeid")] = False  # noqa: B009
 
     def start(self) -> None:
         self.report = None
@@ -49,21 +64,20 @@ class SnapshotSession:
         self._selected_items = {}
         self._assertions = []
         self._extensions = {}
+        self._locations_discovered = defaultdict(set)
 
     def ran_item(self, nodeid: str) -> None:
-        self._selected_items[nodeid] = True
+        if nodeid in self._selected_items:
+            self._selected_items[nodeid] = True
 
     def finish(self) -> int:
         exitstatus = 0
         self.report = SnapshotReport(
-            base_dir=self.base_dir,
+            base_dir=self.pytest_session.config.rootdir,
             collected_items=self._collected_items,
             selected_items=self._selected_items,
             assertions=self._assertions,
-            update_snapshots=self.update_snapshots,
-            warn_unused_snapshots=self.warn_unused_snapshots,
-            include_snapshot_details=self.include_snapshot_details,
-            invocation_args=self._invocation_args,
+            options=self.pytest_session.config.option,
         )
         if self.report.num_unused:
             if self.update_snapshots:
@@ -77,12 +91,17 @@ class SnapshotSession:
 
     def register_request(self, assertion: "SnapshotAssertion") -> None:
         self._assertions.append(assertion)
-        discovered_extensions = {
-            discovered.location: assertion.extension
-            for discovered in assertion.extension.discover_snapshots()
-            if discovered.has_snapshots
-        }
-        self._extensions.update(discovered_extensions)
+
+        test_location = assertion.extension.test_location.filepath
+        extension_class = assertion.extension.__class__
+        if extension_class not in self._locations_discovered[test_location]:
+            self._locations_discovered[test_location].add(extension_class)
+            discovered_extensions = {
+                discovered.location: assertion.extension
+                for discovered in assertion.extension.discover_snapshots()
+                if discovered.has_snapshots
+            }
+            self._extensions.update(discovered_extensions)
 
     def remove_unused_snapshots(
         self,
